#---------------------------------------------------------------------------
#	TITLE : 64 point FFT
#	Ver : 0.0
#	Date : AUG. 25. 1995
#	Author : SunKyu-Kim
#	Program : 64main.sc
#---------------------------------------------------------------------------
#                                                                       #
#       Calling parameters :                                            #
#       Return values :                                                 #
#       Register affected :                                             #
#       Cycle count :                                                   #
#                                                                       #
#-----------------------------------------------------------------------#
#                                                                       #
#	<0> 00	  --> temp.real
#	<1> 01	  --> temp.imag
#
#	<0> 10-2f --> W.REAL                                          	#
#       <1> 10-2f --> W.IMAGE                                         	#
#                                                                       #
#       <0> 30-6f --> X.REAL                                          	#
#       <1> 30-6f --> X.IMAGE                                         	#
#                                                                     	#
#       <0> 70    --> FRAME COUNTER                                   	#
#       <0> 71    --> FFT_data memory pointer                         	#
#
#---------------------------------------------------------------#
#                                                               #
#       Input of FFT()                                          #
#               <RAM0[30-6f]> : 64 real speech samples       	#
#               <RAM0[70]>    : current frame                	#
#               <RAM0[10-2f]> : W.real                       	#
#               <RAM1[00-2f]> : W.imag                       	#
#                                                               #
#       Output of FFT()                                         #
#               <RAM0[30-6F]> : X.real                        	#
#               <RAM1[30-6F]> : X.imag                        	#
#                                                               #
#                                                               #
#       Modified Register                                       #
#               <RAM0[70]>    : current frame number          	#
#               <RAM0[71]>    : pointer of external data mem  	#
#               <RAM0[72]>    : le                            	#
#               <RAM0[73]>    : windx                         	#
#               <RAM0[74]>    : l                             	#
#               <RAM0[75]>    : i                             	#
#               <RAM0[76]>    : wptr                          	#
#               <RAM0[77]>    : j                             	#
#               <RAM0[78]>    : (j)_i                         	#
#               <RAM0[00]>    : temp.real                     	#
#               <RAM1[00]>    : temp.imag                     	#
#               <RAM0[01]>    : tm.real                       	#
#               <RAM1[01]>    : tm.imag                       	#
#                                                               #
#                                                               #
#                                                                       #                                                                       #
#-----------------------------------------------------------------------#

	org  	0000                    # Interrupt vector table for SSP1601M
	                             	#
	dw   	0004   # Reset address  #
	dw   	0004   # INT0 address   #
   	dw   	0004   # INT1 address   #
   	dw   	0004   # INT2 address   #
   
   	org  	0004                    # for SSP1605

#=================<< test vector -- simlulation data >>==================#

Initialize_Processor():
	andi 	00
   	ld   	st, a
   	mod  	f, setop
   	ld   	x,  a
   	ld   	r0, x
   	ld   	r4, x
   	ld   	r1, x

RAM0_Ini:          # initialize RAM0 bank
	ld   	(r0+!), x
   	ld   	a, r0
   	cmpi 	00
   	bra  	z=0, @RAM0_Ini
  
RAM1_Ini:          # initialize RAM1 bank
   	ld   	(r4+!), x
   	ld   	a, r4
   	cmpi 	00
   	bra  	z=0, @RAM1_Ini
   	
# RAM Bank 0_0, 1_0
ldi	r3, 00
ldi 	r7, 00
#ldi	xr0, 0
#ldi	xr4, 0

#-----------------------------------------------------------------#
#                      load_coefficients( )                       #
#-----------------------------------------------------------------#
Twidle_Coeff_Initilize: 

ldi     r0, 10          # Real coeff of twidle factor
ldi     r4, 10          # Image coeff of twidle factor

ldi    (r0+!), 7f62
ldi    (r0+!), 7d8a
ldi    (r0+!), 7a7d
ldi    (r0+!), 7641
ldi    (r0+!), 70e2
ldi    (r0+!), 6a6d
ldi    (r0+!), 62f2
ldi    (r0+!), 5a82
ldi    (r0+!), 5133
ldi    (r0+!), 471c
ldi    (r0+!), 3c56
ldi    (r0+!), 30fb
ldi    (r0+!), 2528
ldi    (r0+!), 18f8
ldi    (r0+!), c8b
ldi    (r0+!), 0
ldi    (r0+!), f375
ldi    (r0+!), e708
ldi    (r0+!), dad8
ldi    (r0+!), cf05
ldi    (r0+!), c3aa
ldi    (r0+!), b8e4
ldi    (r0+!), aecd
ldi    (r0+!), a57e
ldi    (r0+!), 9d0e
ldi    (r0+!), 9593
ldi    (r0+!), 8f1e
ldi    (r0+!), 89bf
ldi    (r0+!), 8583
ldi    (r0+!), 8276
ldi    (r0+!), 809e

ldi    (r4+!), f375
ldi    (r4+!), e708
ldi    (r4+!), dad8
ldi    (r4+!), cf05
ldi    (r4+!), c3aa
ldi    (r4+!), b8e4
ldi    (r4+!), aecd
ldi    (r4+!), a57e
ldi    (r4+!), 9d0e
ldi    (r4+!), 9593
ldi    (r4+!), 8f1e
ldi    (r4+!), 89bf
ldi    (r4+!), 8583
ldi    (r4+!), 8276
ldi    (r4+!), 809e
ldi    (r4+!), 8000
ldi    (r4+!), 809e
ldi    (r4+!), 8276
ldi    (r4+!), 8583
ldi    (r4+!), 89bf
ldi    (r4+!), 8f1e
ldi    (r4+!), 9593
ldi    (r4+!), 9d0e
ldi    (r4+!), a57e
ldi    (r4+!), aecd
ldi    (r4+!), b8e4
ldi    (r4+!), c3aa
ldi    (r4+!), cf05
ldi    (r4+!), dad8
ldi    (r4+!), e708
ldi    (r4+!), f375

# End of twidle factor coeff initilization
	      
# START OF FRAME LOOP
ldi     r0, 70                  # Frame counter as RAM0[70]
ldi     (r0), 00                # clear Frame counter as 00

ldi     r1, 71                  # external fft_data memory pointer
ldi     a, @fft_data
ld      (r1), a                 # Initialize address of external speech_data

#-----------------------------------------------------------------#
#                      load_input_data()                          #
#-----------------------------------------------------------------#
#- loading speech_data from program memory, only for simulation  -#

# Getting 64 samples per 64 samples

Frame_Input():                  # Get a external 64 samples
# Pointer Setting
ldi     r1, 71                  # external speech_data memory pointer
ldi     r2, 30                  # internal One_Frame memory pointer

Loop_Frame_Data:
#-------------- INPUT OF X.REAL ------#
ld      a, ((r1))               # acc <-- current external data 
ld      (r2+!), a               # current internal data mem <-- acc,, INC
andi    00
ld      a, r2
cmpi    a, 70                   # check the final 64'th sample
bra     z=0, @Loop_Frame_Data   # if no,, go ahead data copying 

andi	00
ld	x, a
ldi     r4, 30
Loop_Image_0:
#-------------- RESET OF X.IMAG ------#
ld      (r4+!), x
ld	a, r4
cmpi    a, 70                   # check the final 64'th sample
bra     z=0, @Loop_Image_0   # if no,, go ahead data copying
mod    f, setop                 # OP = 1
mod    f, setie                 # IE = 1

bra    always, @FFT()
include 64fft.sc
include 64br.sc
############################################################

ldi     r0, 70
ld      a, (r0)
addi    a, 01
ld      (r0), a                 # INC.. the frame counter  
cmpi    a, 5
bra     n=1, @Frame_Input()

ld      -, -
ld      -, -

idle:
bra     always, @idle
##################################
include fftdata.sc
##################################
   	
