#----------------------------------------------------------
#  		Least Mean Square
#----------------------------------------------------------
#  * Function :
#   	10th order Least Mean Square routine
#  * Assumption and Constraint :
#
#  * Input Values :
#	RAM1[10~1a] - tap-waight vector w(n)
#	RAM0[10~1a] - Input Buffer u(n)
#
#  * Output Values :
#	RAM1[10~1a] - optimized tap-waight vector w(n+1)
#
#  * Temporal Values :
#	RAM0[00] - Input table address
#	RAM0[01] - Desired data table address
#	RAM0[10] - Counter for repeating p (p=order of filter)
#	RAM0[11] - Temporary buffer
#	RAM1[00] - u (=0.05)
#	RAM1[01] - Error e(n)
#  * Computation time :
#	10-tap filter : maximum 145 cycles
#-----------------------------------------------------------
	org	0000
	dw	0004
	dw	0004
	dw	0004
	dw	0004
	org	0004
    ## Autocorrelation routine
    LMS():
    # initialize
        ldi	(r3|00), @input_table
        ldi	(r3|01), @output_table
        ldi	(r7|00), 0666		# u=0.05
    # LMS main routine ( 10 input example )
        ld	a, st
        andi	a, fff8
        ori	a, 0004
        ld	st, a
    	mod	f, setop
    	ldi	r2, 10			# pointer save buffer
    	ldi	r4, 10			# pointer of coeff.
    	call	always, @lms_loop
    	call	always, @lms_loop
    	call	always, @lms_loop
    	call	always, @lms_loop
    	call	always, @lms_loop
    	call	always, @lms_loop
    	call	always, @lms_loop
    	call	always, @lms_loop
    	call	always, @lms_loop
    	call	always, @lms_loop
    	call	always, @lms_loop
    idle():
    	bra	always, @idle()
    	
    lms_loop:
        ld	a, r2
        ld	r0, a   	# load input buffer pointer
    	ldi	(r3|10), 0a	# the order of w(n)
        ld	x, ((r3|00))	# input u(n) 1 sample
	ld	(r0), x         #
    # estimate output by filtering with w(n)
    	call	always, @predict_filter
        ldi	r4, 10		# load coeff table pointer
	ld	y, ((r3|01))
    	ld	a, y      	# desired value d(n)
    # get e(n) = d(n) - w(n)*u(n)
    	sub	a, (r7|01)	# e(n)=d(n)-w(n)*u(n)
    	ld	(r7|01), a	# save e(n)
        ld	a, r2           # load saved x(n) pointer
        ld	r0, a 		# set pointer to x(n)
        addi    1               # increase r2 by 1
        ld      r2, a           # save new x(n) pointer
    update_w:
    	mld	(r7|01), (r0-)          # (r7|01):e(n), (r0):x(n)
    	ld	(r3|11), p		# 2*x(n)*e(n)
    	mld     (r7|00), (r3|11)	# 2*u*x(n)*e(n)
    	mpya	(r7|00), (r3|11)	# dummy for adding a, p
    	add	a, (r4)                 # w(n)+2*u*x(n)*e(n)
    	ld	(r4+), a		# w(n+1)=w(n)+2*u*x(n)*e(n)
	ld	a, (r3|10)		# counter for repeat p times
	subi	1			# decrease counter
	ld	(r3|10), a		# save reduced counter
	bra	n=0, @update_w
    lms_loop_end:
        ret
        
    predict_filter:
    	mpya	(r4+), (r0-)		# 2*x(n)*w(0)
        mpya    (r4+), (r0-)            # 2*x(n-1)*w(1)
        mpya    (r4+), (r0-)            # 2*x(n-2)*w(2)
        mpya    (r4+), (r0-)            # 2*x(n-3)*w(3)
        mpya    (r4+), (r0-)            # 2*x(n-4)*w(4)
        mpya    (r4+), (r0-)            # 2*x(n-5)*w(5)
        mpya    (r4+), (r0-)            # 2*x(n-6)*w(6)
        mpya    (r4+), (r0-)            # 2*x(n-7)*w(7)
        mpya    (r4+), (r0-)            # 2*x(n-8)*w(8)
        mpya    (r4+), (r0-)            # 2*x(n-9)*w(9)
        mpya    (r4+), (r0-)            # 2*x(n-10)*w(10)
    	ld	(r7|01), a      	# 2*SUM w(k)*x(n-k)
    	ret

input_table:
dw	 0000
dw	 0809
dw	 100a
dw	 17fb
dw	 1fd4
dw	 278d
dw	 2f1e
dw	 367f
dw	 3da9
dw	 4495
dw	 4b3b
dw	 5196
dw	 579e
dw	 5d4e
dw	 629f
dw	 678d
dw	 6c12
dw	 7029
dw	 73d0
dw	 7701
dw	 79bb
dw	 7bf9
dw	 7dba
dw	 7efc
dw	 7fbe
output_table:
dw	 7fff
dw	 7fbe
dw	 7efc
dw	 7dba
dw	 7bf9
dw	 79bb
dw	 7701
dw	 73d0
dw	 7029
dw	 6c12
dw	 678d
dw	 629f
dw	 5d4e
dw	 579e
dw	 5196
dw	 4b3b
dw	 4495
dw	 3da9
dw	 367f
dw	 2f1e
dw	 278d
dw	 1fd4
dw	 17fb
dw	 100a
dw	 0809
dw	 0000
dw       f7f7
dw       eff6
dw       e805
dw       e02c
dw       d873
dw       d0e2
dw       c981
dw       c257
dw       bb6b
dw       b4c5
dw       ae6a
dw       a862
dw       a2b2
dw       9d61
dw       9873
dw       93ee
dw       8fd7
dw       8c30
dw       88ff
dw       8645
dw       8407
dw       8246
dw       8104
dw       8042
dw       8001
dw       8042
dw       8104
dw       8246
dw       8407
dw       8645
dw       88ff
dw       8c30
dw       8fd7
dw       93ee
dw       9873
dw       9d61
dw       a2b2
dw       a862
dw       ae6a
dw       b4c5
dw       bb6b
dw       c257
dw       c981
dw       d0e2
dw       d873
dw       e02c
dw       e805
dw       eff6
dw       f7f7
dw	 0000
dw	 0809
dw	 100a
dw	 17fb
dw	 1fd4
dw	 278d
dw	 2f1e
dw	 367f
dw	 3da9
dw	 4495
dw	 4b3b
dw	 5196
dw	 579e
dw	 5d4e
dw	 629f
dw	 678d
dw	 6c12
dw	 7029
dw	 73d0
dw	 7701
dw	 79bb
dw	 7bf9
dw	 7dba
dw	 7efc
dw	 7fbe
dw	 7fff
dw	 7fbe
dw	 7efc
dw	 7dba
dw	 7bf9
dw	 79bb
dw	 7701
dw	 73d0
dw	 7029
dw	 6c12
dw	 678d
dw	 629f
dw	 5d4e
dw	 579e
dw	 5196
dw	 4b3b
dw	 4495
dw	 3da9
dw	 367f
dw	 2f1e
dw	 278d
dw	 1fd4
dw	 17fb
dw	 100a
dw	 0809
dw	 0000
dw       f7f7
dw       eff6
dw       e805
dw       e02c
dw       d873
dw       d0e2
dw       c981
dw       c257
dw       bb6b
dw       b4c5
dw       ae6a
dw       a862
dw       a2b2
dw       9d61
dw       9873
dw       93ee
dw       8fd7
dw       8c30
dw       88ff
dw       8645
dw       8407
dw       8246
dw       8104
dw       8042
dw       8001
dw       8042
dw       8104
dw       8246
dw       8407
dw       8645
dw       88ff
dw       8c30
dw       8fd7
dw       93ee
dw       9873
dw       9d61
dw       a2b2
dw       a862
dw       ae6a
dw       b4c5
dw       bb6b
dw       c257
dw       c981
dw       d0e2
dw       d873
dw       e02c
dw       e805
dw       eff6
dw       f7f7
dw	 0000
