#------------------------------------------------------
#    		Normalization
#------------------------------------------------------
#  * Function :
# 	First, absolute values of inputs are made, and then,
#	the maximum amplitude is found and all the inputs are 
#	shifted by (the number of the zero before the valid bit
#	in the maximum amplitude) - 1.
#
#  * Assumption & Constraint
#
#  * Input Variables
#	RAM0[10~1f] - input buffer
#
#  * Output Variables
#	RAM0[10~1f] - output buffer
#	RAM0[11] - exponent
#
#  * Temporal Variables
#	RAM0[00] - maximum value
#	RAM0[01] - counter
#  * Computation time :
#	8 sample normalization : maximum 203 cycles
#---------------------------------------------------------
	org	0000
	dw	0004
	dw	0004
	dw	0004
	dw	0004
	org	0004
    # initialize
    	and	a, -
    	ldi	r0, 10			# pointer of input buffer
    	ld	stack, r0		# save pointer of input...
    	ldi	(r3|00), @input_table
    	ldi	(r3|01), 10
    	subi	1
    	ld 	(r3|10), a		# counter = input # - 1
    	ldi	a, 10           	# counter of input number
    load_input:
    	ld	x, ((r3|00))
    	ld      (r0+), x		# load inputs to RAN0[10~]
        subi	1
        bra	z=0, @load_input
        
    # find max input
    	ldi	(r3|00), 0		# initial maximum
     	ld 	r0, stack		# base addr. of inputs
    find_max_x:
    	ld      a, (r3|01)
    	subi	1			# decrease counter
        ld      (r3|01), a
        bra     n=1, @find_exponent
    	ld	a, (r0+)                # load 1 input
    	mod	always, abs		# absolute value of input
    	cmp	a, (r3|00)
    	bra	n=1, @find_max_x
	ld	(r3|00), a
        bra	always, @find_max_x
    find_exponent:
    	ld	a, (r3|00)
    	cmpi	a, 100
    	bra	n=0, @over_9
        cmpi	10
        bra	n=0, @over_5
        cmpi	04
        bra	n=0, @over_3
        cmpi	02
        bra	n=0, @over_2
        ldi	(r3|11), 0e
        bra     always, @get_normalized_value
    over_2:
        ldi	(r3|11), 0d
        bra     always, @get_normalized_value
    over_3:
	cmpi	08
	bra	n=0, @over_4
	ldi	(r3|11), 0c
        bra     always, @get_normalized_value
    over_4:
	ldi	(r3|11), 0b
        bra     always, @get_normalized_value
    over_5:
    	cmpi	40
    	bra	n=0, @over_7
    	cmpi    20
    	bra	n=0, @over_6
    	ldi     (r3|11), 0a
        bra     always, @get_normalized_value
    over_6:
    	ldi     (r3|11), 09
        bra     always, @get_normalized_value
    over_7:
        cmpi	80
        bra	n=0, @over_8
    	ldi     (r3|11), 08
        bra     always, @get_normalized_value
    over_8:
    	ldi     (r3|11), 07
        bra     always, @get_normalized_value
    over_9:
    	cmpi	a, 1000
    	bra	n=0, @over_13
    	cmpi	a, 0400
    	bra	n=0, @over_11
    	cmpi	a, 0200
    	bra	n=0, @over_10
    	ldi     (r3|11), 06
        bra     always, @get_normalized_value
    over_10:
    	ldi     (r3|11), 05
        bra     always, @get_normalized_value
    over_11:
        cmpi	a, 0800
        bra	n=0, @over_12
    	ldi     (r3|11), 04
        bra     always, @get_normalized_value
    over_12:
    	ldi     (r3|11), 03
        bra     always, @get_normalized_value
    over_13:
    	cmpi	a, 4000
    	bra	n=0, @over_15
    	cmpi	a, 2000
    	bra	n=0, @over_14
    	ldi     (r3|11), 02
        bra     always, @get_normalized_value
    over_14:
    	ldi	(r3|11), 01
        bra     always, @get_normalized_value
    over_15:
	ldi	(r3|11), 00
	bra	always, @normalize_end
    
    get_normalized_value:
        ldi	a, @shift_table
        add	a, (r3|11)
        ld	y, (a)                  # load shift value to y
    normalize_loop:
        andi	00
	ld	x, (r0)			# load input to x
        add	a, p
	ld      (r0+), ext7		# save normalized value
	ld	a, (r3|10)
	subi	1
	ld	(r3|10), a
	bra     n=0, @normalize_loop
    normalize_end:
    	bra	always, @normalize_end
    	
    shift_table:
    	dw	0000
    	dw	0001		# shfl1
        dw      0002            # shfl2
        dw      0004            # shfl3
        dw      0008            # shfl4
        dw      0010            # shfl5
        dw      0020            # shfl6
        dw      0040            # shfl7
        dw      0080            # shfl8
        dw      0100            # shfl9
        dw      0200            # shfl10
        dw      0400            # shfl11
        dw      0800            # shfl12
        dw      1000            # shfl13
        dw      2000            # shfl14
        dw      4000            # shfl15
    input_table:
        dw	0001
        dw	0002
        dw	0003
        dw	0004
        dw	0005
        dw	C001
        dw	0007
        dw	0008
        dw	0009
        dw	000a
        dw	000b
        dw	000c
        dw	000d
        dw	000e
        dw	000f
        dw	003F


